"use strict";
/**
 * Reclass doc generator
 *
 * @author Jiri Hybek <jiri@hybek.cz>
 * @license Apache-2.0 (c) 2017 Jiri Hybek
 */
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var fsExtra = require("fs-extra");
var pug = require("pug");
var Util_1 = require("./Util");
/**
 * Renderer class
 */
var Renderer = (function () {
    /**
     * Renderer constructor
     *
     * @param config Renderer config
     * @param logger Logger facility
     */
    function Renderer(config, logger) {
        /** Inventory item cache - path => fingerprint */
        this.cache = {};
        this.outputDir = config.outputDir;
        this.classesDir = config.classesDir || '/classes';
        this.nodesDir = config.nodesDir || '/nodes';
        this.sourcePostfix = config.sourcePostfix || '.source';
        this.templateDir = config.templateDir;
        this.assetsSrcDir = config.assetsSrcDir || '/assets/dist';
        this.assetsOutDir = config.assetsOutDir || '/assets';
        this.globals = config.globals || {};
        this.options = config.options || {
            pretty: true
        };
        this.title = config.title || 'Reclass reference';
        this.logger = logger;
    }
    /**
     * Prepares navigation tree for template
     *
     * @param index Inventory index
     * @param path Relative path
     */
    Renderer.prototype.prepareTree = function (index, path) {
        if (path === void 0) { path = ''; }
        //Create nav item
        var item = {
            type: (index.classes['init'] ? 'class' : 'directory'),
            label: index.name,
            link: './' + path + "/index.html",
            items: []
        };
        //Add classes
        for (var i in index.classes)
            if (i !== 'init')
                item.items.push({
                    type: 'class',
                    label: index.classes[i].name,
                    link: '.' + path + "/class." + index.classes[i].name + ".html",
                    items: null
                });
        for (var i in index.docs)
            if (i !== 'README')
                item.items.push({
                    type: 'document',
                    label: index.docs[i].name,
                    link: '.' + path + "/doc." + index.docs[i].name + ".html",
                    items: null
                });
        for (var i in index.dirs)
            item.items.push(this.prepareTree(index.dirs[i], path + "/" + index.dirs[i].name));
        //Sort
        item.items.sort(function (a, b) {
            return a.label.localeCompare(b.label);
        });
        //Return
        return item;
    };
    /**
     * Renders source page
     *
     * @param filename Filename
     */
    Renderer.prototype.renderFile = function ($filename) {
    };
    /**
     * Renders class page
     *
     * @param invClass Inventory resolved class
     */
    Renderer.prototype.renderClass = function ($invClass) {
    };
    /**
     * Renders index page
     *
     * @param index Inventory index
     */
    Renderer.prototype.renderIndex = function (index) {
        var outputFilename = this.outputDir + '/index.html';
        try {
            this.logger.info("Rendering index to '" + outputFilename + "'...");
            var html = pug.renderFile(this.templateDir + '/index.pug', Util_1.merge(this.globals, this.options, {
                title: this.title,
                navTree: {
                    classes: this.prepareTree(index.dirs['classes'], this.classesDir),
                    nodes: this.prepareTree(index.dirs['nodes'], this.nodesDir)
                }
            }));
            fs.writeFileSync(outputFilename, html, { encoding: 'utf-8' });
        }
        catch (err) {
            this.logger.warn("Failed to render index:", err);
        }
    };
    /**
     * Renders main page
     * @param index Inventory index
     */
    Renderer.prototype.renderMain = function (index) {
        var outputFilename = this.outputDir + '/main.html';
        try {
            this.logger.info("Rendering index to '" + outputFilename + "'...");
            var html = pug.renderFile(this.templateDir + '/main.pug', Util_1.merge(this.globals, this.options));
            fs.writeFileSync(outputFilename, html, { encoding: 'utf-8' });
        }
        catch (err) {
            this.logger.warn("Failed to render index:", err);
        }
    };
    /**
     * Copy assets from template to output for given directory
     *
     * @param path Path
     */
    Renderer.prototype.copyAssets = function (path) {
        var realPath = this.templateDir + this.assetsSrcDir + path;
        var dstPath = this.outputDir + this.assetsOutDir + path;
        this.logger.debug("Checking assets in '" + realPath + "'...");
        //Check output dir
        if (!fs.existsSync(dstPath)) {
            this.logger.info("Creating directory '" + dstPath + "'...");
            fs.mkdirSync(dstPath);
        }
        //Read dir
        var files = fs.readdirSync(realPath);
        for (var i in files) {
            var file = files[i];
            if (file == "." || file == ".." || file.substr(0, 1) == ".")
                continue;
            var srcFilename = this.templateDir + this.assetsSrcDir + path + "/" + file;
            var dstFilename = this.outputDir + this.assetsOutDir + path + "/" + file;
            var assetId = "__assets__" + path + "/" + file;
            var stat = fs.statSync(srcFilename);
            if (stat.isDirectory()) {
                try {
                    this.copyAssets(path + "/" + file);
                }
                catch (err) {
                    this.logger.error("Failed to create asset directory '" + srcFilename + "':", err);
                }
            }
            else {
                this.logger.debug("Asset file '" + srcFilename + "' up to date, skiping...");
                if (this.cache[assetId] == String(stat.mtime.getTime()))
                    continue;
                this.logger.info("Copying asset file '" + srcFilename + "' to '" + dstFilename + "'...");
                //Copy file
                try {
                    fsExtra.copySync(srcFilename, dstFilename, {
                        overwrite: true
                    });
                }
                catch (err) {
                    this.logger.error("Failed to copy asset file '" + srcFilename + "':", err);
                }
                this.cache[assetId] = String(stat.mtime.getTime());
            }
        }
    };
    /**
     * Renders all items in inventory
     *
     * @param index Inventory index
     */
    Renderer.prototype.render = function (index, treeHash) {
        //Check output directory
        if (!fs.existsSync(this.outputDir))
            fsExtra.mkdirpSync(this.outputDir);
        //Copy assets
        this.copyAssets("");
        //Render nav
        if (this.cache['__tree__'] != treeHash)
            this.renderIndex(index);
        //Render main
        //if(this.cache[index.path] !== index.fingerprint)
        //	this.renderMain(index);
    };
    return Renderer;
}());
exports.Renderer = Renderer;
